local addonName, L = ...;
local TeamCache = ArenaLiveCore:GetHandler("TeamCache");
local UnitTracker = ArenaLiveCore:GetHandler("UnitTracker");
local MatchStatistic = ArenaLiveCore:GetHandler("MatchStatistic");
-- Current Unit Frame DropDown Displayed Name:
ALSPEC_DROPDOWN_DISPLAY_NAME_UNIT = nil;

-- Binding Localisation:
BINDING_HEADER_ARENALIVESPECTATOR_TITLE = L["BINDING_ARENALIVESPECTATOR_TITLE"];
BINDING_NAME_ARENALIVESPECTATOR_TOGGLE = L["BINDING_ARENALIVESPECTATOR_TOGGLE"];
BINDING_NAME_ARENALIVESPECTATOR_TOGGLE_PLAYER = L["ARENALIVESPECTATOR_TOGGLE_PLAYER"];
BINDING_NAME_ARENALIVESPECTATOR_TOGGLE_COUNTDOWN = L["BINDING_ARENALIVESPECTATOR_TOGGLE_COUNTDOWN"];

-- Team Colours:
ArenaLiveSpectator.teamColours = 
{
	["Left"] = {0, 0.5, 1, 1},
	["Right"] = {1, 0.19, 0, 1},
}

-- Default Settings:
local DEFAULT_SETTINGS =
{
	-- MAIN DEFAULTS:
	["Scale"] = 100,
	["GameMode"] = 3,
	["DisableQueueButton"] = true;
	["TimeLimit/Minutes"] = 15,
	["TimeLimit/Seconds"] = 0,
	["Auras/TournamentFilter"] = true,
	
	-- COUNTDOWN DEFAULTS:
	["Countdown/Broadcast"] = true,
	["Countdown/BroadcastChannel"] = "PARTY",
	["Countdown/Time"] = 10,

	-- SCOREBOARD DEFAULTS:
	["Scoreboard/Show"] = true,
	["Scoreboard/TournamentIcon"] = "Interface\\AddOns\\ArenaLiveSpectator\\Textures\\StandardTournamentIcon",
	
	-- TEAM DEFAULTS:
	["Teams/Left/Name"] = "Left Team",
	["Teams/Left/Tag"] = "LT",
	["Teams/Left/Icon"] = "",
	["Teams/Left/Score"] = 0,
	["Teams/Left/Member1"] = "",
	["Teams/Left/Member2"] = "",
	["Teams/Left/Member3"] = "",
	
	["Teams/Right/Name"] = "Right Team",
	["Teams/Right/Tag"] = "RT",
	["Teams/Right/Icon"] = "",
	["Teams/Right/Score"] = 0,
	["Teams/Right/Member1"] = "",
	["Teams/Right/Member2"] = "",
	["Teams/Right/Member3"] = "",

	-- TEAM CACHE:
	["TeamCache"] = {},
	
	-- NAME ALIASES:
	["NameAliases"] = {},
	
	-- COOLDOWN TRACKER SETTINGS:
	["CooldownTracker/MaxIcons"] = 18,
	["CooldownTracker/Icon/XOffset"] = 0,
	["CooldownTracker/Icon/YOffset"] = 2,	
	
	-- FRAME SETTING DEFAULTS:
	["SideFrames/Enabled"] = true,
	["SideFrames/Scale"] = 100,
	["SideFrames/Portrait/Type"] = "class",
	["SideFrames/Icon1/Type"] = "trinket",
	["SideFrames/Icon2/Type"] = "interruptOrDispel",
	["SideFrames/StatusBars/ReverseFill"] = false,
	["SideFrames/HealthBar/ColourMode"] = "class",
	["SideFrames/HealthBar/Text"] = "%CURR_SHORT% / %MAX_SHORT% (%PERCENT_SHORT%%)",
	["SideFrames/PowerBar/Text"] = "%CURR_SHORT% / %MAX_SHORT% (%PERCENT_SHORT%%)",
	["SideFrames/NameText/ColourMode"] = "none",
	["SideFrames/AuraFrame/Enabled"] = true,
	["SideFrames/AuraFrame/PlayerIconSize"] = 25,
	["SideFrames/AuraFrame/NormalIconSize"] = 25,
	["SideFrames/AuraFrame/GrowUpwards"] = false,
	["SideFrames/AuraFrame/MaxShownBuffs"] = 8,
	["SideFrames/AuraFrame/MaxShownDebuffs"] = 16,
	["SideFrames/CastHistory/Enabled"] = true,
	["SideFrames/CastHistory/IconSize"] = 21,
	["SideFrames/CastHistory/Direction"] = "UP",
	["SideFrames/CastHistory/MaxShownIcons"] = 5,
	
	["TargetFrame/Enabled"] = true,
	["TargetFrame/Scale"] = 100,
	["TargetFrame/Portrait/Type"] = "class",
	["TargetFrame/Icon1/Type"] = "trinket",
	["TargetFrame/Icon2/Type"] = "racial",
	["TargetFrame/StatusBars/ReverseFill"] = false,
	["TargetFrame/HealthBar/ColourMode"] = "class",
	["TargetFrame/HealthBar/Text"] = "%CURR_SHORT% / %MAX_SHORT% (%PERCENT_SHORT%%)",
	["TargetFrame/PowerBar/Text"] = "%CURR_SHORT% / %MAX_SHORT% (%PERCENT_SHORT%%)",
	["TargetFrame/NameText/ColourMode"] = "none",
	["TargetFrame/AuraFrame/Enabled"] = true,
	["TargetFrame/AuraFrame/PlayerIconSize"] = 32,
	["TargetFrame/AuraFrame/NormalIconSize"] = 32,
	["TargetFrame/AuraFrame/GrowUpwards"] = false,
	["TargetFrame/AuraFrame/MaxShownBuffs"] = 8,
	["TargetFrame/AuraFrame/MaxShownDebuffs"] = 8,
	["TargetFrame/CastBar/Enabled"] = true,
	["TargetFrame/CastHistory/Enabled"] = true,
	["TargetFrame/CastHistory/IconSize"] = 28,
	["TargetFrame/CastHistory/Direction"] = "RIGHT",
	["TargetFrame/CastHistory/MaxShownIcons"] = 9,

	["TargetOfTargetFrame/Enabled"] = true,
	["TargetOfTargetFrame/Scale"] = 100,
	["TargetOfTargetFrame/Portrait/Type"] = "class",
	["TargetOfTargetFrame/Icon1/Type"] = "trinket",
	["TargetOfTargetFrame/Icon2/Type"] = "racial",
	["TargetOfTargetFrame/StatusBars/ReverseFill"] = false,
	["TargetOfTargetFrame/HealthBar/ColourMode"] = "class",
	["TargetOfTargetFrame/HealthBar/Text"] = "%CURR_SHORT% / %MAX_SHORT% (%PERCENT_SHORT%%)",
	["TargetOfTargetFrame/PowerBar/Text"] = "%CURR_SHORT% / %MAX_SHORT% (%PERCENT_SHORT%%)",
	["TargetOfTargetFrame/NameText/ColourMode"] = "none",
	["TargetOfTargetFrame/AuraFrame/Enabled"] = true,
	["TargetOfTargetFrame/AuraFrame/PlayerIconSize"] = 32,
	["TargetOfTargetFrame/AuraFrame/NormalIconSize"] = 32,
	["TargetOfTargetFrame/AuraFrame/GrowUpwards"] = false,
	["TargetOfTargetFrame/AuraFrame/MaxShownBuffs"] = 8,
	["TargetOfTargetFrame/AuraFrame/MaxShownDebuffs"] = 8,
	["TargetOfTargetFrame/CastBar/Enabled"] = true,
	["TargetOfTargetFrame/CastHistory/Enabled"] = true,
	["TargetOfTargetFrame/CastHistory/IconSize"] = 28,
	["TargetOfTargetFrame/CastHistory/Direction"] = "LEFT",
	["TargetOfTargetFrame/CastHistory/MaxShownIcons"] = 9,
};

function ArenaLiveSpectator:Initialise()
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("UPDATE_BATTLEFIELD_STATUS");
	self:SetScript("OnEvent", self.OnEvent);
end

function ArenaLiveSpectator:OnEvent(event, ...)
	
	local arg1 = ...;
	
	if ( event == "ADDON_LOADED" and arg1 == addonName ) then
		if ( not ArenaLiveCore:DatabaseExists(addonName) ) then
			ArenaLiveCore:AddDatabase(addonName, DEFAULT_SETTINGS, false);
		--else
			--ArenaLiveCore:UpdateDatabase(addonName, DEFAULT_SETTINGS);
		end
		
		-- Change font styles for non latin sign systems:
		ArenaLiveSpectator:ChangeFonts();
		
		-- Option Panel:
		ALSpec_Options:Initialise();
		
		-- Scoreboard:
		ALSpec_ScoreBoard:Initialise();
		
		-- Initialise TeamCache Handler:
		TeamCache:Initialise();
		
		-- Initialse Team Menu:
		ALSpec_TeamMenu:Initialise();
	
		-- Initialise CountDown Frame:
		ALSpec_CountDown:Initialise();
	
		-- Initialise Match Statistic:
		ALSpec_MatchStatistic:Initialise()
	
		-- Initialise Cooldown Tracker:
		ArenaLiveSpectator:InitialiseCooldownTracker("Left");
		ArenaLiveSpectator:InitialiseCooldownTracker("Right");
		
		-- Initialise Side Frames:
		for i = 1, 3 do
			local frame = _G["ALSpec_LeftFrame"..i];
			ArenaLiveSpectator:InitialiseSideFrame(frame, "Left");
			
			frame =  _G["ALSpec_RightFrame"..i];
			ArenaLiveSpectator:InitialiseSideFrame(frame, "Right");
		end
		
		-- Initialise Target and Target-of-Target:
		ArenaLiveSpectator:InitialiseTargetFrame(ALSpec_TargetFrame);
		ALSpec_TargetFrame:SetUnit("target");
		ArenaLiveSpectator:InitialiseTargetOfTargetFrame(ALSpec_TargetOfTargetFrame);
		ALSpec_TargetOfTargetFrame:SetUnit("targettarget");
		
		-- Message for the client that loading process is complete:
		ArenaLiveCore:Message(L["ADDON_LOADED_MESSAGE"], "normal");
	elseif ( event == "ALSPEC_TEAM_NAME_UPDATE" ) then
		ALSpec_ScoreBoard:UpdateTeamName(arg1);
	elseif ( event == "ALSPEC_TEAM_TAG_UPDATE" ) then
		ALSpec_ScoreBoard:UpdateTeamName(arg1);
	elseif ( event == "ALSPEC_TEAM_ICON_UPDATE" ) then
		ALSpec_ScoreBoard:UpdateTeamIcon(arg1);
	elseif ( event == "ALSPEC_TEAM_MEMBER_UPDATE" ) then
		local team, memberID = ...;
		local frame = _G["ALSpec_"..team.."Frame"..memberID];
		frame:CheckUnit();
	elseif ( event == "ALSPEC_COUNTDOWN_FINISHED" ) then
		ArenaLiveSpectator.inMatch = true;
		ALSpec_ScoreBoard:StartTimer();
		ALSpec_MatchStatistic:OnMatchStart();
		MatchStatistic:PLAYER_ENTERING_WORLD();
	elseif ( event == "ALSPEC_MATCH_FINISHED" ) then
		ArenaLiveSpectator.inMatch = false;

		-- Set up match statistic:
		ALSpec_MatchStatistic:OnMatchEnd();
		MatchStatistic:PLAYER_ENTERING_WORLD();
		
		-- Reset Cooldown tracker:
		ALSpec_CooldownTrackerLeft:Reset();
		ALSpec_CooldownTrackerRight:Reset();
		
	elseif ( event == "ALSPEC_AVAILABLE_UNITS_UPDATE" ) then
		ArenaLiveSpectator:CheckAllSideFrameUnits();
	elseif ( event == "UPDATE_BATTLEFIELD_STATUS" ) then
		-- From Blizzard's PVPHeler.lua: status, mapName, teamSize, registeredMatch, suspendedQueue, queueType, gameType, role
		-- GetMaxBattlefieldID();
		-- GetBattlefieldWinner(): 0 =  green team, 1 = gold team, 255 = draw.
		-- IsActiveBattlefieldArena(): returns isArena (1/nil), isRegistered (1/nil)
		-- GetBattlefieldArenaFaction(): 0 =  green team, 1 = gold team;
		-- UPDATE_BATTLEFIELD_STATUS: "active" for joining arena and when BG ends.  Use GetBattlefieldWinner to know when it is over.	
	
		local bgID = ...;
		local status, mapName, teamSize, registeredMatch, suspendedQueue, queueType, gameType, role = GetBattlefieldStatus(bgID);
		-- Battlefield is active, we've got a winner and are currently in a match. This means that the match has ended normally:
		if ( ArenaLiveSpectator.inMatch and status == "active" and GetBattlefieldWinner() ) then
			ALSpec_MatchStatistic:Show();
			ALSpec_ScoreBoard:StopTimer();
			ArenaLiveSpectator:OnEvent("ALSPEC_MATCH_FINISHED");		
		end
	end
end

function ArenaLiveSpectator:GetLocalisation()
	return L;
end

function ArenaLiveSpectator:GetUnitName(unit)
	
	local name, realm;
	
	if ( unit == "player" ) then
		name = UnitName(unit);
		realm = GetRealmName();
		name = name.."-"..realm;
		return name;
	end
	
	if ( UnitGUID(unit) ) then
		name, realm = UnitName(unit);
		
		-- If the unit is from the 
		if ( not realm or realm == "" ) then
			realm = GetRealmName();
		end		
		
		name = name.."-"..realm;
		
		return name;
	else
		return nil;
	end
	
end

function ArenaLiveSpectator:Toggle()
	if ( not InCombatLockdown() ) then
		
		local showScoreBoard = ArenaLiveCore:GetDBEntry(addonName, "Scoreboard/Show");
		
		if ( self:IsShown() ) then
			self:Hide();
			
			-- Migrate ScoreBoard to UIParent frame if needed:
			if ( ALSpec_ScoreBoard:GetParent() == self ) then
				ALSpec_ScoreBoard:SetParent(UIParent);
			end
			ALSpec_ScoreBoard:SetMode(true);
			UIParent:Show();
		else
			UIParent:Hide();
			ALSpec_ScoreBoard:SetMode(false);
			-- Migrate ScoreBoard to ArenaLiveSpectator frame if needed:
			if ( showScoreBoard ) then
				ALSpec_ScoreBoard:SetParent(ArenaLiveSpectator);
			end
			
			self:Show();	
		end

		-- Show/Hide Player Model:
		ConsoleExec("ShowPlayer");
	else
		ArenaLiveCore:Message(L["CANNOT_SHOW_OR_HIDE_UI_COMBATLOCKDOWN"], "error");
	end
end

-- Some language clients need other Fonts to show every information correctly (e.g. Russian and Korean).
function ArenaLiveSpectator:ChangeFonts()
	
	local fontStyle = L["FONTSTYLE"];
	if ( fontStyle ~= "Fonts\\FRIZQT__.TTF" ) then
		ALSpec_NameFont:SetFont(fontStyle, 16);
		ALSpec_ScoreBoardFont:SetFont(fontStyle, 28);
		ALSpec_ScoreBoardTimeFont:SetFont(fontStyle, 16);
		ALSPec_CooldownTrackerFont:SetFont(fontStyle, 10, "OUTLINE");
	end
end

ArenaLiveSpectator:Initialise();