local addonName, L = ...;

local MatchStatistic = ArenaLiveCore:AddHandler("MatchStatistic", "EventCore");
local NameText = ArenaLiveCore:GetHandler("NameText");

-- Register Events:
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_SPELL_HEAL");
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_SPELL_DAMAGE");
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_SPELL_PERIODIC_HEAL");
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_SPELL_PERIODIC_DAMAGE");
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_SWING_DAMAGE");
--MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_PARTY_KILL");
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_SPELL_AURA_APPLIED");
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_SPELL_AURA_REMOVED");
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_SPELL_AURA_REFRESH");
MatchStatistic:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED_UNIT_DIED");
MatchStatistic:RegisterEvent("PLAYER_ENTERING_WORLD");
MatchStatistic:RegisterEvent("UNIT_AURA");
MatchStatistic:RegisterEvent("UNIT_PET");
MatchStatistic:RegisterEvent("ARENA_OPPONENT_UPDATE");

-- Store all Statistics in one table:
local MatchStatisticFrames = {};

-- *** FRAME FUNCTIONS: ***
local function Start(self)
	self.record = true;
end

local function Stop(self)
	self.record = false;
end

local function AddUnit(self, unit)
	-- DO NOT ADD PETS HERE!
	-- Get the unit's name to fit ArenaLiveSpectator's and NameText handler's name system:
	if ( not unit ) then
		return;
	end
	
	local name, realm = UnitName(unit);
	local displayedName;
	if ( name ) then
		if ( not realm ) then
			realm = GetRealmName();
		end
		
		name = name.."-"..realm;
		
		-- Get displayed Name:
		local alias = NameText:GetNameAlias(nil, name);
		if ( alias ) then
			name = alias;
		else
			name =  UnitName(unit);
		end				
		
	else
		-- TODO: ERROR;
		return;
	end

	-- Check if the unit has a pet:
	local unitType = string.match(unit, "^([a-z]+)[0-9]+$") or unit;
	local unitNumber = tonumber(string.match(unit, "^[a-z]+([0-9]+)$"));		
	local petUnit;
	if ( unitType == "player" ) then
		petUnit = "pet";
	elseif ( unitNumber ) then
		petUnit = unitType.."pet"..unitNumber;
	else
		petUnit = unitType.."pet";
	end
	
	-- Get Class Filename:
	local _, class = UnitClass(unit);
	
	-- Get GUID to set as table key:
	local guid = UnitGUID(unit);
	
	-- Set pet to pet table:
	local petGuid = UnitGUID(petUnit);
	if ( petGuid ) then
		self.pets[petGuid] = guid;
	end	
	
	-- Now fill the table with information:
	if ( not self.players[guid] ) then
		self.players[guid] = {};
	else
		table.wipe(self.players[guid]);
	end
	
	self.players[guid]["name"] = name;
	self.players[guid]["class"] = class;
	self.players[guid]["damage"] = 0;
	self.players[guid]["healing"] = 0;
	self.players[guid]["absorb"] = 0;
	self.players[guid]["kills"] = 0;
	self.players[guid]["highestDamage"] = 0; -- will be a string in the format: "%d\n(ABILITY_NAME)";
	self.players[guid]["highestDamageAbility"] = "";
	self.players[guid]["timeCCed"] = 0;
	
end

local function RemoveUnit(self, unit)

	local isPlayer = UnitIsPlayer(unit); -- Only track player as pets get an entry in the player's table.
	if ( not isPlayer ) then
		return;
	end
	
	local guid = UnitGUID(unit);
	
end

local function Report(self)

	for playerGUID, infoTable in pairs(self.players) do
		local name = tostring(infoTable["name"]);
		local damage = tostring(infoTable["damage"]);
		local healing = tostring(infoTable["healing"]);
		local absorb = tostring(infoTable["absorb"]);
		local kills = tostring(infoTable["kills"]);
		local highestDamage = tostring(infoTable["highestDamage"]);
		local highestDamageAbility = tostring(infoTable["highestDamageAbility"]);
		local timeCCed = tostring(infoTable["timeCCed"]);
		
		print(name..":");
		print("     dmg: "..damage.."; heal: "..healing.."; absorb: "..absorb.."; kills:"..kills.."; highest: "..highestDamage.." ("..highestDamageAbility.."); timeCCed: "..timeCCed);
		
	end

end

local function Reset(self)
	table.wipe(self.players);
	table.wipe(self.pets);
	self.record = false;
end
 
-- *** HANDLER FUNCTIONS: ***
function MatchStatistic:AddFrame(frame)
	
	MatchStatisticFrames[frame] = true;
	
	-- Base function(s):
	frame.Start = Start;
	frame.Stop = Stop;
	frame.AddUnit = AddUnit;
	frame.RemoveUnit = RemoveUnit;
	frame.Report = Report;
	frame.Reset = Reset;

	-- Set base values:
	frame.players = {};
	frame.pets = {};
	frame.record = false;
	
	-- Set Scripts:
	frame:SetScript("OnUpdate", OnUpdate);
end

function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_SPELL_HEAL(...)

	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end		
	
	local amount = select(15, ...);
	local overhealing = select(16, ...);

	if ( overhealing ) then
		amount = amount - overhealing;
	end
	
	-- Iterate through all statistics:
	for statistic, value in pairs(MatchStatisticFrames) do
		if ( statistic.record ) then
			if ( statistic.players[sourceGUID] ) then
				statistic.players[sourceGUID]["healing"] = statistic.players[sourceGUID]["healing"] + amount;
			end
			if ( statistic.pets[sourceGUID] ) then
				local petOwnerGUID = statistic.pets[sourceGUID];
				statistic.players[petOwnerGUID]["healing"] = statistic.players[petOwnerGUID]["healing"] + amount;
			end
		end
	end	
	
end

function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_SPELL_DAMAGE(...)
	
	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end	
	
	local spellName = select(13, ...);
	local amount = select(15, ...);
	local overkill = select(16, ...);
	local resisted = select(18, ...);
	local blocked = select(19, ...);
	-- 20 would be absorbed, but we count hitting on absorb as damage in this statistic.
	
	-- Get destGUID unit type to filter killing blows on non players:
	local destGUID = select(8, ...);
	local destGUIDUnitType = bit.band(tonumber(destGUID:sub(5,5), 16), 0x7);

	local rawAmount = amount;
	
	if ( overkill ) then
		amount = amount - overkill;
	end

	if ( resisted ) then
		amount = amount - resisted;
	end

	if ( blocked ) then
		amount = amount - blocked;
	end

	-- Iterate through all statistics:
	for statistic, value in pairs(MatchStatisticFrames) do
		if ( statistic.record ) then
			if ( statistic.players[sourceGUID] ) then
				statistic.players[sourceGUID]["damage"] = statistic.players[sourceGUID]["damage"] + amount;
				
				-- Check if this was a killing blow:
				if ( overkill > 0 and destGUIDUnitType == 0 ) then
					statistic.players[sourceGUID]["kills"] = statistic.players[sourceGUID]["kills"] + 1;
				end
				
				-- Use raw amount for highest damage:
				if ( statistic.players[sourceGUID]["highestDamage"] < rawAmount ) then
					statistic.players[sourceGUID]["highestDamage"] = rawAmount;
					statistic.players[sourceGUID]["highestDamageAbility"] = spellName;
				end
			end
			if ( statistic.pets[sourceGUID] ) then
				local petOwnerGUID = statistic.pets[sourceGUID];
				statistic.players[petOwnerGUID]["damage"] = statistic.players[petOwnerGUID]["damage"] + amount;

				-- Check if this was a killing blow:
				if ( overkill > 0 and destGUIDUnitType == 0 ) then
					statistic.players[petOwnerGUID]["kills"] = statistic.players[petOwnerGUID]["kills"] + 1;
				end
				
				-- Use raw amount for highest damage:
				if ( statistic.players[petOwnerGUID]["highestDamage"] < rawAmount ) then
					statistic.players[petOwnerGUID]["highestDamage"] = rawAmount;
					statistic.players[petOwnerGUID]["highestDamageAbility"] = spellName;
				end
			end
		end
	end
end

function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_SPELL_PERIODIC_HEAL(...)
	
	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end		
	
	local amount = select(15, ...);
	local overhealing = select(16, ...);

	if ( overhealing ) then
		amount = amount - overhealing;
	end
	
	-- Iterate through all statistics:
	for statistic, value in pairs(MatchStatisticFrames) do
		if ( statistic.record ) then
			if ( statistic.players[sourceGUID] ) then
				statistic.players[sourceGUID]["healing"] = statistic.players[sourceGUID]["healing"] + amount;
			end
			if ( statistic.pets[sourceGUID] ) then
				local petOwnerGUID = statistic.pets[sourceGUID];
				statistic.players[petOwnerGUID]["healing"] = statistic.players[petOwnerGUID]["healing"] + amount;
			end
		end
	end	

end

function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_SPELL_PERIODIC_DAMAGE(...)
	
	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end	
	
	local spellName = select(13, ...);
	local amount = select(15, ...);
	local overkill = select(16, ...);
	local resisted = select(18, ...);
	local blocked = select(19, ...);
	-- Again: 20 would be absorbed, but we count hitting on absorb as damage in this statistic.
	
	-- Get destGUID unit type to filter killing blows on non players:
	local destGUID = select(8, ...);
	local destGUIDUnitType = bit.band(tonumber(destGUID:sub(5,5), 16), 0x7);	
	
	local rawAmount = amount;
	
	if ( overkill > 0 ) then
		amount = amount - overkill;
	end

	if ( resisted ) then
		amount = amount - resisted;
	end

	if ( blocked ) then
		amount = amount - blocked;
	end

	-- Iterate through all statistics:
	for statistic, value in pairs(MatchStatisticFrames) do
		if ( statistic.record ) then
			if ( statistic.players[sourceGUID] ) then
				statistic.players[sourceGUID]["damage"] = statistic.players[sourceGUID]["damage"] + amount;

				-- Check if this was a killing blow:
				if ( overkill > 0 and destGUIDUnitType == 0 ) then
					statistic.players[sourceGUID]["kills"] = statistic.players[sourceGUID]["kills"] + 1;
				end
				
				-- Use raw amount for highest damage:
				if ( statistic.players[sourceGUID]["highestDamage"] < rawAmount ) then
					statistic.players[sourceGUID]["highestDamage"] = rawAmount;
					statistic.players[sourceGUID]["highestDamageAbility"] = spellName;
				end
			end
			if ( statistic.pets[sourceGUID] ) then
				local petOwnerGUID = statistic.pets[sourceGUID];
				statistic.players[petOwnerGUID]["damage"] = statistic.players[petOwnerGUID]["damage"] + amount;

				-- Check if this was a killing blow:
				if ( overkill > 0 and destGUIDUnitType == 0 ) then
					statistic.players[petOwnerGUID]["kills"] = statistic.players[petOwnerGUID]["kills"] + 1;
				end
				
				-- Use raw amount for highest damage:
				if ( statistic.players[petOwnerGUID]["highestDamage"] < rawAmount ) then
					statistic.players[petOwnerGUID]["highestDamage"] = rawAmount;
					statistic.players[petOwnerGUID]["highestDamageAbility"] = spellName;
				end
			end
		end
	end
	
end

function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_SWING_DAMAGE(...)

	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end	

	local amount = select(12, ...);
	local overkill = select(13, ...);
	local resisted = select(15, ...);
	local blocked = select(16, ...);
	-- 17 would be absorbed, but we count hitting on absorb as damage in this statistic.

	-- Get destGUID unit type to filter killing blows on non players:
	local destGUID = select(8, ...);
	local destGUIDUnitType = bit.band(tonumber(destGUID:sub(5,5), 16), 0x7);	
	
	if ( overkill > 0 ) then
		amount = amount - overkill;
	end

	if ( resisted ) then
		amount = amount - resisted;
	end

	if ( blocked ) then
		amount = amount - blocked;
	end

	-- Iterate through all statistics:
	for statistic, value in pairs(MatchStatisticFrames) do
		if ( statistic.record ) then
			if ( statistic.players[sourceGUID] ) then
				statistic.players[sourceGUID]["damage"] = statistic.players[sourceGUID]["damage"] + amount;
				
				-- Check if this was a killing blow:
				if ( overkill > 0 and destGUIDUnitType == 0 ) then
					statistic.players[sourceGUID]["kills"] = statistic.players[sourceGUID]["kills"] + 1;
				end
			end
			
			if ( statistic.pets[sourceGUID] ) then
				local petOwnerGUID = statistic.pets[sourceGUID];
				statistic.players[petOwnerGUID]["damage"] = statistic.players[petOwnerGUID]["damage"] + amount;
				
				-- Check if this was a killing blow:
				if ( overkill > 0 and destGUIDUnitType == 0 ) then
					statistic.players[petOwnerGUID]["kills"] = statistic.players[petOwnerGUID]["kills"] + 1;
				end
			end
		end
	end
	
end

--[[ Absorbs can only be tracked with regard to their source via auras.
	 AURA_APPLIED and AURA_REFRESH: Already contain the amount of absorb that is applied.
	 AURA_REMOVED: Contains the amount that is left after the aura is removed.
	 So what I'll do here is to simply put every applied absorb into a table and adding it
	 to the sources as soon as the absorb has run out.
]]--
local AbsorbWaitingForRemove = {};
function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_SPELL_AURA_APPLIED(...)

	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end	
	
	local destGUID = select(8, ...);
	local spellID = select(12, ...);
	local auraType = select(15, ...);
	local amount = select(16, ...);

	if ( auraType == "BUFF" and amount ) then
		local aid = sourceGUID.."-"..destGUID.."-"..spellID;
		AbsorbWaitingForRemove[aid] = amount;
	end
	
end

function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_SPELL_AURA_REMOVED(...)

	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end	
	
	local destGUID = select(8, ...);
	local spellID = select(12, ...);
	local auraType = select(15, ...);
	local amount = select(16, ...);

	if ( auraType == "BUFF" and amount ) then
		local aid = sourceGUID.."-"..destGUID.."-"..spellID;
		if ( AbsorbWaitingForRemove[aid] ) then
			amount = AbsorbWaitingForRemove[aid] - amount
			AbsorbWaitingForRemove[aid] = nil;

			-- Iterate through all statistics:
			for statistic, value in pairs(MatchStatisticFrames) do
				if ( statistic.record ) then
					if ( statistic.players[sourceGUID] ) then
						statistic.players[sourceGUID]["absorb"] = statistic.players[sourceGUID]["absorb"] + amount;
					end
					if ( statistic.pets[sourceGUID] ) then
						local petOwnerGUID = statistic.pets[sourceGUID];
						statistic.players[petOwnerGUID]["absorb"] = statistic.players[petOwnerGUID]["absorb"] + amount;
					end
				end
			end
		end
	end	
	
end

function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_SPELL_AURA_REFRESH(...)

	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end	
	
	local destGUID = select(8, ...);
	local spellID = select(12, ...);
	local auraType = select(15, ...);
	local amount = select(16, ...);

	if ( auraType == "BUFF" and amount ) then
		local aid = sourceGUID.."-"..destGUID.."-"..spellID;
		AbsorbWaitingForRemove[aid] = amount;
	end	
	
end

function MatchStatistic:COMBAT_LOG_EVENT_UNFILTERED_PARTY_KILL(...)

	local sourceGUID = select(4, ...);
	
	if ( not sourceGUID ) then
		return;
	end		

	-- Iterate through all statistics:
	for statistic, value in pairs(MatchStatisticFrames) do
		if ( statistic.record ) then
			if ( statistic.players[sourceGUID] ) then
				statistic.players[sourceGUID]["kills"] = statistic.players[sourceGUID]["kills"] + 1;
			end
			if ( statistic.pets[sourceGUID] ) then
				local petOwnerGUID = statistic.pets[sourceGUID];
				statistic.players[petOwnerGUID]["kills"] = statistic.players[petOwnerGUID]["kills"] + 1;
			end
		end
	end	
	
end

local MAX_DEBUFFS = 40;
local CCWaitingForRemove = {};
function MatchStatistic:UNIT_AURA(...)
	local unit = ...;
	local guid = UnitGUID(unit);

	local isInCC = nil;
	for index = 1, MAX_DEBUFFS do
		local name, _, _, _, _, _, _, _, _, _, spellID = UnitDebuff(unit, index);
		if ( ArenaLiveCore.spellDB["diminishingReturns"][spellID] ) then
			isInCC = true;
			break;
		end
	end

	if ( isInCC ) then
		-- If there already is an entry then it definitely has been created before:
		if ( not CCWaitingForRemove[guid] ) then
			CCWaitingForRemove[guid] = GetTime();
		end
	else
		if ( CCWaitingForRemove[guid] ) then
			local duration = GetTime() - CCWaitingForRemove[guid];
			CCWaitingForRemove[guid] = nil;
			
			-- Iterate through all statistics:
			for statistic, value in pairs(MatchStatisticFrames) do
				if ( statistic.record ) then
					if ( statistic.players[guid] ) then
						statistic.players[guid]["timeCCed"] = statistic.players[guid]["timeCCed"] + duration;
					end
				end
			end
		end
	end
end

--[[ Since UNIT_PET also fires for removed pets, I store all pet unitIDs with their current
	 GUID in a table in order to being able to delete pets that have beend dismissed/killed etc.]]--
local UnitPetGUIDCache = {};
function MatchStatistic:UNIT_PET(...)
	local unit = ...;
	local guid = UnitGUID(unit);
	local petUnit;
	local unitType = string.match(unit, "^([a-z]+)[0-9]+$") or unit;
	local unitNumber = tonumber(string.match(unit, "^[a-z]+([0-9]+)$"));
	
	if ( unitType == "player" ) then
		petUnit = "pet";
	elseif ( unitNumber ) then
		petUnit = unitType.."pet"..unitNumber;
	else
		petUnit = unitType.."pet";
	end
	
	local petGUID = UnitGUID(petUnit);
	local oldPetGUID = UnitPetGUIDCache[petUnit];
	
	-- Update entries:
	for statistic, value in pairs(MatchStatisticFrames) do
		if ( statistic.record ) then
			if ( statistic.players[guid] ) then
				if ( petGUID ) then
					statistic.pets[petGUID] = guid;
				elseif ( oldPetGUID ) then
					statistic.pets[oldPetGUID] = nil;
				end
			end
		end
	end
	
	-- Update Cache:
	UnitPetGUIDCache[petUnit] = petGUID;
end

function MatchStatistic:ARENA_OPPONENT_UPDATE(petUnit, state)

	local unitType = string.match(petUnit, "^([a-z]+)[0-9]+$") or petUnit;
	local unitNumber = tonumber(string.match(petUnit, "^[a-z]+([0-9]+)$"));
	
	if ( unitType == "arenapet" and ( state == "seen" or state == "cleared" or state == "destoryed" ) ) then
	
		local unit = "arena"..unitNumber;
		local guid = UnitGUID(unit);
		local petGUID = UnitGUID(petUnit);
		local oldPetGUID = UnitPetGUIDCache[unit];
		
		-- Update entries:
		for statistic, value in pairs(MatchStatisticFrames) do
			if ( statistic.record ) then
				if ( statistic.players[guid] ) then
					if ( petGUID ) then
						statistic.pets[petGUID] = guid;
					elseif ( oldPetGUID ) then
						statistic.pets[oldPetGUID] = nil;
					end
				end
			end
		end
	
		-- Update Cache:
		UnitPetGUIDCache[petUnit] = petGUID;
	end
end

function MatchStatistic:PLAYER_ENTERING_WORLD()

	-- Wipe cache tables that wouldn't be otherwise:
	table.wipe(CCWaitingForRemove);
	table.wipe(AbsorbWaitingForRemove);

end

function MatchStatistic:OnEvent(event, ...)
	
	if ( MatchStatistic[event] ) then
		MatchStatistic[event](self, ...)
	end

end