local addonName, L = ...;

local TeamCache = ArenaLiveCore:GetHandler("TeamCache");
local ImportExport = ArenaLiveCore:GetHandler("ImportExport");

-- Local reference to the saved variable table containing the teams:
local teamDB;

function Button_ImportTeamCache()
	local import = ALSpec_OptionsTeamCacheScrollFrameImportExportEditBox:GetText();
	
	if ( string.find(import, "^(teamDB =)") ) then
		local returnTable = ImportExport:LuaDataToTable(import);
		for team, infoTable in pairs(returnTable.teamDB) do
			teamDB[team] = infoTable;
		end
	else
		ArenaLiveCore:Message(L["NAME_ALIAS_ERROR_WRONG_INPUT"], "error");
	end
	ALSpec_OptionsTeamCacheScrollFrameImportExportEditBox:SetText("");
	ALSpec_OptionsTeamCacheScrollFrameImportExportEditBox:SetCursorPosition(0);
end

function Button_ExportTeamCache()
	ALSpec_OptionsTeamCacheScrollFrameImportExportEditBox:SetText("teamDB = " ..ImportExport:TableToLuaData(teamDB));
	ALSpec_OptionsTeamCacheScrollFrameImportExportEditBox:SetFocus();
	ALSpec_OptionsTeamCacheScrollFrameImportExportEditBox:HighlightText();
end


function ALSpec_OptionsTeamCache:Initialise()
	local prefix = self:GetName();
	
	-- Get team cache:
	teamDB = ArenaLiveCore:GetDBEntry(addonName, "TeamCache");
	
	-- Add Option Panel:
	self.name = L["TEAM_CACHE_PANEL_NAME"];
	self.parent = L["ADDON_PANEL_NAME"];
	InterfaceOptions_AddCategory(self); 
	
	-- Set Text:
	_G[prefix.."Title"]:SetText(L["TEAM_CACHE_PANEL_TITLE"]);

	-- Initialise Team Cache Options:
	TeamCache:InitialiseDropdown(ALSpec_OptionsTeamCacheDeleteTeamDropDown, function(button) TeamCache:DeleteTeam(button.value); end, L["TEAM_CACHE_DELETE_DROPDOWN_TITLE"]);
	ArenaLiveOptions:InitialiseButton(_G[prefix.."ResetButton"], addonName, nil, function() TeamCache:DeleteAll() end, L["TEAM_CACHE_RESET_BUTTON_TEXT"])
	ArenaLiveOptions:InitialiseButton(ALSpec_OptionsTeamCacheScrollFrameImportButton, addonName, nil, Button_ImportTeamCache, L["TEAM_CACHE_IMPORT_BUTTON_TEXT"]);
	ArenaLiveOptions:InitialiseButton(ALSpec_OptionsTeamCacheScrollFrameExportButton, addonName, nil, Button_ExportTeamCache, L["TEAM_CACHE_EXPORT_BUTTON_TEXT"]);	
end